#!/usr/bin/python3
from dbus.mainloop.glib import DBusGMainLoop
from gi.repository import GLib
import configparser
import time
import logging
import os
import dbus
import sys

TIMESTAMP_PATH="/var/lib/kylin-software-properties/template/kylin-source-status"

def get_timestamp():
    global timeStamp
    config=configparser.ConfigParser(allow_no_value=True)
    config.optionxform = str
    config.read(TIMESTAMP_PATH)
    time_value=time.localtime(int(config.get("Server","UpdateTime")))
    logging.debug(("获取软件源时间戳：%s"),time_value)
    timeStamp="自动备份："+time.strftime("%Y-%m-%d %H:%M:%S",time_value)+" "+config.get("Server","UpdateTime")
    return timeStamp

class KylinBackupManager:
    def __init__(self) -> None:
        DBusGMainLoop(set_as_default=True)
        self.loop = GLib.MainLoop()
        self.system_bus = dbus.SystemBus()
        self.backup_proxy = self.system_bus.get_object('com.kylin.backup','/')
        self.backup_interface = dbus.Interface(self.backup_proxy,dbus_interface='com.kylin.backup.manager')
        self.success = False

    def mount_backup_partition(self):
        return self.backup_interface.Mount_backup_partition()

    def get_backup_state(self):
        return self.backup_interface.getBackupState()

    def get_backup_comment_for_systemupdate(self):
        return self.backup_interface.getBackupCommentForSystemUpdate()
    
    def auto_backup_for_system_update_noreturn(self,timeStamp,create_note,inc_note,userName,uid):
        self.backup_interface.autoBackUpForSystemUpdate_noreturn(timeStamp,create_note,inc_note,userName,uid)
        return

    def ConnectToSignals(self):
        def backup_start_handler(result):
            logging.debug("backup start result:%d"%result)
            if result == 31 or result == 30:
                logging.debug("backup start success")
            else:
                logging.error("backup start failed")
                os._exit(1)
            
        def backup_result_handler(result):
            if result:
                logging.debug("backup success,quiting main loop")
                self.loop.quit()  
            else:
                logging.error("backup failed")
                os._exit(1)
                 
        def send_rate_handler(sta,pro):
            logging.debug(("receive backup_rate_signal_handler 状态:%d 进度:%d"),sta,pro)
            if pro == 100:
                logging.debug("backup finished, quiting mainloop")
                self.loop.quit()           

        self.backup_proxy.connect_to_signal('sendStartBackupResult',backup_start_handler,
                                        dbus_interface='com.kylin.backup.manager')
        self.backup_proxy.connect_to_signal('sendBackupResult',backup_result_handler,
                                        dbus_interface='com.kylin.backup.manager')  
        self.backup_proxy.connect_to_signal('sendRate',send_rate_handler,
                                            dbus_interface='com.kylin.backup.manager')                            
        return

    def RunMainloop(self):
        logging.info("backup manager:running mainloop")
        self.loop.run()

    def QuitMainloop(self):
        logging.info("backup manager:quiting mainloop")
        self.loop.quit()


def Backup():
    # do backup
    kylin_backup_manager = KylinBackupManager()
    backup_partition_status = kylin_backup_manager.mount_backup_partition()
    logging.info("backup partition status:%d"%backup_partition_status)
    if backup_partition_status not in [0,5]:
        logging.error("backup partition error:%d"%backup_partition_status)
        # return UnattendedUpgradesResult(False,"backup partition error")
        return False
    status_code,result = kylin_backup_manager.get_backup_state()
    
    if result == 0 and status_code == 99:
        pass
    else:
        logging.error("backup state error:",status_code,result)
        # return UnattendedUpgradesResult(False,"backup state error")
        return False
    #node_name,node_status = kylin_backup_manager.get_backup_comment_for_systemupdate()
    ts = get_timestamp()
    kylin_backup_manager.ConnectToSignals()
    create_note = "系统升级新建备份"
    inc_note="系统升级增量备份"
    userName="root"
    uid=os.getuid()
    kylin_backup_manager.auto_backup_for_system_update_noreturn(ts,create_note,inc_note,userName,uid)
    kylin_backup_manager.RunMainloop()
    return True

if __name__ =="__main__":
    logdir = "/var/log/kylin-unattended-upgrades/"
    # setup logging
    level = logging.INFO
    if not os.path.exists('/var/lib/unattended-upgrades'):
        os.makedirs('/var/lib/unattended-upgrades')
    if not os.path.exists(logdir):
        os.makedirs(logdir)
    logfile = os.path.join(logdir, "unattended-upgrades-shutdown.log")
    logging.basicConfig(format='%(asctime)s-%(name)s-%(levelname)s-%(message)s',level=logging.DEBUG,filename=logfile)
    Backup()
    sys.exit(0)